# Copyright (C) 2021 MicroSys Electronics GmbH
# Author: Kay Potthoff <kay.potthoff@microsys.de>
# Modified: Sriraman Ravi <sriraman.ravi@microsys.de>

create_sparse_file() {

    rootfs=$1
    sdcard=$2

    if [ -z "${rootfs}" ]; then
        bberror "${rootfs} is undefined."
        exit 1
    fi

    bbdebug 1 "rootfs=${rootfs}"

    ROOTFS_SIZE=$(stat -L --printf="%s" ${rootfs})

    bbdebug 1 "ROOTFS_SIZE=${ROOTFS_SIZE}"

    SDCARD_SIZE=$(expr ${ROOTFS_SIZE} + 256 \* 1024)

    PART_START=$(expr 64 \* 2048)

    bbdebug 1 "sdcard=${sdcard}"
    bbdebug 1 "SDCARD_SIZE=${SDCARD_SIZE}"
    bbdebug 1 "PART_START=${PART_START}s"

    # Initialize a sparse file
    dd if=/dev/zero of=${sdcard} bs=1k count=0 seek=${SDCARD_SIZE}

    echo $PART_START
}

partition_sparse_file() {

    sdcard=$1
    part_start=$2

    parted --script ${sdcard} mklabel msdos
    parted ${sdcard} mkpart primary ext4 ${part_start}s 100%
    bbdebug 1 $(parted ${sdcard} print)
}

IMAGE_CMD:tfa-sdcard () {

    PART_START=$(create_sparse_file ${SDCARD_ROOTFS} ${SDCARD})

    partition_sparse_file ${SDCARD} ${PART_START}

    if [ ${MACHINE} = "mpxls1028crx07" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
    fi

    if [ ${MACHINE} = "mpxls1046ram2gbcrx05" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/fsl_fman_ucode_ls1046_r1.0_106_4_18.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=18432
    fi
    if [ ${MACHINE} = "mpxls1046ram4gbcrx05" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/fsl_fman_ucode_ls1046_r1.0_106_4_18.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=18432
    fi
    if [ ${MACHINE} = "mpxls1046ram4gb2nandcrx05" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/fsl_fman_ucode_ls1046_r1.0_106_4_18.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=18432
    fi
    if [ ${MACHINE} = "mpxls1046ram8gbcrx05" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/fsl_fman_ucode_ls1046_r1.0_106_4_18.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=18432
    fi
    if [ ${MACHINE} = "mpxls1046crx06" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/fsl_fman_ucode_ls1046_r1.0_106_4_18.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=18432
    fi

    # We are doing the same for every mpxls1043 machine - We could use regex for this
    if [ "${MACHINE}" = "mpxls1043ac1000msatacrx05" ] \
    || [ "${MACHINE}" = "mpxls1043ac1000pciecrx05" ] \
    || [ "${MACHINE}" = "mpxls1043ac1600msatacrx05" ] \
    || [ "${MACHINE}" = "mpxls1043ac1600pciecrx05" ] \
    || [ "${MACHINE}" = "mpxls1043ac1600pcie2nandcrx05" ] \
    || [ "${MACHINE}" = "mpxls1043ac1600pcie2gbnandcrx05" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/fsl_fman_ucode_ls1043_r1.1_106_4_18.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=18432
		dd if=${DEPLOY_DIR_IMAGE}/boot/${QE_UCODE} conv=notrunc,fsync of=${SDCARD} bs=512 seek=18944
    fi
    if [ ${MACHINE} = "mpxls1088crx05" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
		dd if=${DEPLOY_DIR_IMAGE}/mc_app/mc.itb conv=notrunc,fsync of=${SDCARD} bs=512 seek=20480
		dd if=${DEPLOY_DIR_IMAGE}/dpl-eth-crx05.0x12_0x0D.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=26624
		dd if=${DEPLOY_DIR_IMAGE}/dpc-crx05.0x12-0x0D.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=28672
    fi
    if [ ${MACHINE} = "mpxls1088crx06" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
		dd if=${DEPLOY_DIR_IMAGE}/mc_app/mc.itb conv=notrunc,fsync of=${SDCARD} bs=512 seek=20480
		dd if=${DEPLOY_DIR_IMAGE}/dpl-eth-crx06-2tsn.0x19_0x0C.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=26624
		dd if=${DEPLOY_DIR_IMAGE}/dpc-crx06-2tsn.0x19-0x0C.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=28672
    fi
    if [ ${MACHINE} = "mpxls1088crx06qsgmii" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
		dd if=${DEPLOY_DIR_IMAGE}/mc_app/mc.itb conv=notrunc,fsync of=${SDCARD} bs=512 seek=20480
		dd if=${DEPLOY_DIR_IMAGE}/dpl-eth-crx06-2qsgmii.0x1D_0x0C.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=26624
		dd if=${DEPLOY_DIR_IMAGE}/dpc-crx06-2qsgmii.0x1D-0x0C.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=28672
    fi
    if [ ${MACHINE} = "mpxlx2160acrx08" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/ddr-phy/fip_ddr_all.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=16384
		dd if=${DEPLOY_DIR_IMAGE}/mc_app/mc.itb conv=notrunc,fsync of=${SDCARD} bs=512 seek=20480
		dd if=${DEPLOY_DIR_IMAGE}/dpl-eth.19.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=26624
		dd if=${DEPLOY_DIR_IMAGE}/dpc-usxgmii.crx08.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=28672
    fi
    if [ ${MACHINE} = "mpxlx2160aacu" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/ddr-phy/fip_ddr_all.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=16384
		dd if=${DEPLOY_DIR_IMAGE}/mc_app/mc.itb conv=notrunc,fsync of=${SDCARD} bs=512 seek=20480
		dd if=${DEPLOY_DIR_IMAGE}/dpl-eth.11.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=26624
		dd if=${DEPLOY_DIR_IMAGE}/dpc.acu.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=28672
    fi
    if [ ${MACHINE} = "mpxlx2160acrx08ai" ]; then
        dd if=${DEPLOY_DIR_IMAGE}/atf/bl2_sd.pbl conv=notrunc,fsync of=${SDCARD} bs=512 seek=8
        dd if=${DEPLOY_DIR_IMAGE}/atf/fip_uboot.bin of=${SDCARD} conv=notrunc,fsync bs=512 seek=2048
        dd if=${DEPLOY_DIR_IMAGE}/ddr-phy/fip_ddr_all.bin conv=notrunc,fsync of=${SDCARD} bs=512 seek=16384
		dd if=${DEPLOY_DIR_IMAGE}/mc_app/mc.itb conv=notrunc,fsync of=${SDCARD} bs=512 seek=20480
		dd if=${DEPLOY_DIR_IMAGE}/dpl-eth.19.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=26624
		dd if=${DEPLOY_DIR_IMAGE}/dpc-usxgmii.crx08.dtb conv=notrunc,fsync of=${SDCARD} bs=512 seek=28672
    fi

    dd if=${SDCARD_ROOTFS} of=${SDCARD} conv=notrunc,fsync seek=${PART_START} bs=512
}

IMAGE_TYPEDEP:tfa-sdcard:append = " ${@d.getVar('SDCARD_ROOTFS', 1).split('.')[-1]}"

IMAGE_CMD:ramdisk () {
    mkimage -A arm -T ramdisk -d ${IMGDEPLOYDIR}/${IMAGE_NAME}${IMAGE_NAME_SUFFIX}.cpio.gz ${IMGDEPLOYDIR}/${IMAGE_NAME}${IMAGE_NAME_SUFFIX}.ramdisk
}
